float4  pixelSize;
float2  horzTapOffs[7];		// Gauss blur horizontal texture coordinate offsets
float2  vertTapOffs[7];		// Gauss blur vertical texture coordinate offsets
float4	TexelWeight[7];
float2  texelSize;
float	ExposureLevel;

texture RenderMap;
sampler RenderMapSampler = sampler_state
{
   Texture = (RenderMap);
   MinFilter = LINEAR;
   MagFilter = LINEAR;
   MipFilter = LINEAR;
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture FullResMap;
sampler FullResMapSampler = sampler_state
{
   Texture = (FullResMap);
   MinFilter = LINEAR;
   MagFilter = LINEAR;
   MipFilter = LINEAR;
   AddressU  = Clamp;
   AddressV  = Clamp;
};

// -------------------------------------------------------------
// Output channels
// -------------------------------------------------------------
struct VS_OUTPUTScaleBuffer
{
    float4 Pos	: POSITION;
	float2 Tex	: TEXCOORD0;	
};

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------
VS_OUTPUTScaleBuffer VSScaleBuffer(float4 Pos : POSITION)
{
    VS_OUTPUTScaleBuffer Out = (VS_OUTPUTScaleBuffer)0;        
    Out.Pos.xy = Pos.xy + pixelSize.xy;
	Out.Pos.z = 0.5f;
	Out.Pos.w = 1.0f;
	
	Out.Tex = float2(0.5f, -0.5f) * Pos.xy + 0.5f.xx;	
	
    return Out;
}

// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
float4 PSScaleBuffer( float2 Tex :TEXCOORD0): COLOR				
{
    return tex2D(RenderMapSampler, Tex);
}


// -------------------------------------------------------------
// Output channels
// -------------------------------------------------------------
struct VS_OUTPUT_GaussX
{
    float4 Pos		: POSITION;
    float2 Tap0		: TEXCOORD0;
    float2 Tap1		: TEXCOORD1;    
    float2 Tap2		: TEXCOORD2;
    float2 Tap3		: TEXCOORD3;
    float2 Tap1Neg	: TEXCOORD4;
    float2 Tap2Neg	: TEXCOORD5;        
    float2 Tap3Neg	: TEXCOORD6;        
};

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------
VS_OUTPUT_GaussX VSGaussX(float4 Pos      : POSITION)
{
    VS_OUTPUT_GaussX Out = (VS_OUTPUT_GaussX)0;        
    Out.Pos.xy = Pos.xy + pixelSize.xy;
	Out.Pos.z = 0.5f;
	Out.Pos.w = 1.0f;
	
	float2 Tex = float2(0.5f, -0.5f) * Pos.xy + 0.5f.xx;	
	
	Out.Tap0 = Tex;
	Out.Tap1 = Tex + horzTapOffs[1];	
	Out.Tap2 = Tex + horzTapOffs[2];
	Out.Tap3 = Tex + horzTapOffs[3];
	Out.Tap1Neg = Tex - horzTapOffs[1];
	Out.Tap2Neg = Tex - horzTapOffs[2];			
	Out.Tap3Neg = Tex - horzTapOffs[3];			

    return Out;
}


// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
float4 PSGaussX( float2 Tap0	: TEXCOORD0,
				float2 Tap1		: TEXCOORD1,    
				float2 Tap2		: TEXCOORD2,
				float2 Tap3		: TEXCOORD3,
				float2 Tap1Neg	: TEXCOORD4,
				float2 Tap2Neg	: TEXCOORD5,        
				float2 Tap3Neg	: TEXCOORD6) : COLOR0
{
	float4 Color[7];
	float4 ColorSum = 0.0f;	

	// sample inner taps
	Color[0]  = tex2D(RenderMapSampler, Tap0); 
	Color[1]  = tex2D(RenderMapSampler, Tap1);
	Color[2]  = tex2D(RenderMapSampler, Tap1Neg);
	Color[3]  = tex2D(RenderMapSampler, Tap2); 
	Color[4]  = tex2D(RenderMapSampler, Tap2Neg);
	Color[5]  = tex2D(RenderMapSampler, Tap3);
	Color[6]  = tex2D(RenderMapSampler, Tap3Neg); 
	
	ColorSum = Color[0] * TexelWeight[0];
	ColorSum += Color[1] * TexelWeight[1];	
	ColorSum += Color[2] * TexelWeight[1];
	ColorSum += Color[3] * TexelWeight[2];
	ColorSum += Color[4] * TexelWeight[2];			
	ColorSum += Color[5] * TexelWeight[3];	
	ColorSum += Color[6] * TexelWeight[3];	
	
	// compute texture coordinates for other taps
	float2 Tap4 = Tap0 + horzTapOffs[4];
	float2 Tap5 = Tap0 + horzTapOffs[5];
	float2 Tap6 = Tap0 + horzTapOffs[6];
	float2 Tap4Neg = Tap0 - horzTapOffs[4];
	float2 Tap5Neg = Tap0 - horzTapOffs[5];
	float2 Tap6Neg = Tap0 - horzTapOffs[6];
	
	// sample outer taps
	Color[0]  = tex2D(RenderMapSampler, Tap4); 
	Color[1]  = tex2D(RenderMapSampler, Tap4Neg);
	Color[2]  = tex2D(RenderMapSampler, Tap5); 
	Color[3]  = tex2D(RenderMapSampler, Tap5Neg); 
	Color[4]  = tex2D(RenderMapSampler, Tap6); 
	Color[5]  = tex2D(RenderMapSampler, Tap6Neg); 

	ColorSum += Color[0] * TexelWeight[4];
	ColorSum += Color[1] * TexelWeight[4];
	ColorSum += Color[2] * TexelWeight[5];
	ColorSum += Color[3] * TexelWeight[5];
	ColorSum += Color[4] * TexelWeight[6];
	ColorSum += Color[5] * TexelWeight[6];	

	return ColorSum;				
}


// -------------------------------------------------------------
// Output channels
// -------------------------------------------------------------
struct VS_OUTPUT_GaussY
{
    float4 Pos		: POSITION;
    float2 Tap0		: TEXCOORD0;
    float2 Tap1		: TEXCOORD1;    
    float2 Tap2		: TEXCOORD2;
    float2 Tap3		: TEXCOORD3;
    float2 Tap1Neg	: TEXCOORD4;
    float2 Tap2Neg	: TEXCOORD5;        
    float2 Tap3Neg	: TEXCOORD6;        
};

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------
VS_OUTPUT_GaussY VSGaussY(float4 Pos      : POSITION)
{
    VS_OUTPUT_GaussY Out = (VS_OUTPUT_GaussY)0;        
    Out.Pos.xy = Pos.xy + pixelSize.xy;
	Out.Pos.z = 0.5f;
	Out.Pos.w = 1.0f;
	
	float2 Tex = float2(0.5f, -0.5f) * Pos.xy + 0.5f.xx;	

	Out.Tap0 = Tex;
	Out.Tap1 = Tex + vertTapOffs[1];	
	Out.Tap2 = Tex + vertTapOffs[2];
	Out.Tap3 = Tex + vertTapOffs[3];
	Out.Tap1Neg = Tex - vertTapOffs[1];
	Out.Tap2Neg = Tex - vertTapOffs[2];			
	Out.Tap3Neg = Tex - vertTapOffs[3];			

    return Out;
}

// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
float4 PSGaussY( float2 Tap0	: TEXCOORD0,
				float2 Tap1		: TEXCOORD1,    
				float2 Tap2		: TEXCOORD2,
				float2 Tap3		: TEXCOORD3,
				float2 Tap1Neg	: TEXCOORD4,
				float2 Tap2Neg	: TEXCOORD5,        
				float2 Tap3Neg	: TEXCOORD6) : COLOR0
{
	float4 Color[7];
	float4 ColorSum = 0.0f;
	
	// sample inner taps
	Color[0] = tex2D(RenderMapSampler, Tap0);
	Color[1] = tex2D(RenderMapSampler, Tap1);
	Color[2] = tex2D(RenderMapSampler, Tap1Neg);
	Color[3] = tex2D(RenderMapSampler, Tap2);
	Color[4] = tex2D(RenderMapSampler, Tap2Neg);
	Color[5] = tex2D(RenderMapSampler, Tap3);
	Color[6] = tex2D(RenderMapSampler, Tap3Neg);
	
	ColorSum = Color[0] * TexelWeight[0];
	ColorSum += Color[1] * TexelWeight[1];	
	ColorSum += Color[2] * TexelWeight[1];
	ColorSum += Color[3] * TexelWeight[2];
	ColorSum += Color[4] * TexelWeight[2];			
	ColorSum += Color[5] * TexelWeight[3];	
	ColorSum += Color[6] * TexelWeight[3];	

	// compute texture coordinates for other taps
	float2 Tap4 = Tap0 + vertTapOffs[4];
	float2 Tap5 = Tap0 + vertTapOffs[5];
	float2 Tap6 = Tap0 + vertTapOffs[6];
	float2 Tap4Neg = Tap0 - vertTapOffs[4];
	float2 Tap5Neg = Tap0 - vertTapOffs[5];
	float2 Tap6Neg = Tap0 - vertTapOffs[6];
	
	// sample outer taps
	Color[0] = tex2D(RenderMapSampler, Tap4);
	Color[1] = tex2D(RenderMapSampler, Tap4Neg);
	Color[2] = tex2D(RenderMapSampler, Tap5);
	Color[3] = tex2D(RenderMapSampler, Tap5Neg);
	Color[4] = tex2D(RenderMapSampler, Tap6);
	Color[5] = tex2D(RenderMapSampler, Tap6Neg);									

	ColorSum += Color[0] * TexelWeight[4];
	ColorSum += Color[1] * TexelWeight[4];
	ColorSum += Color[2] * TexelWeight[5];
	ColorSum += Color[3] * TexelWeight[5];
	ColorSum += Color[4] * TexelWeight[6];
	ColorSum += Color[5] * TexelWeight[6];	
	
	return ColorSum;				
}


// -------------------------------------------------------------
// Output channels
// -------------------------------------------------------------
struct VS_OUTPUTScreen
{
    float4 Pos			: POSITION;	
	float2 Tex			: TEXCOORD0;
};

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------
VS_OUTPUTScreen VSScreen(float4 Pos      : POSITION)
{
    VS_OUTPUTScreen Out = (VS_OUTPUTScreen)0;        
    Out.Pos.xy	= Pos.xy; // + pixelSize.xy;
	Out.Pos.z	= 0.5f;
	Out.Pos.w	= 1.0f;
	
	Out.Tex		= float2(0.5f, -0.5f) * Pos.xy + 0.5f.xx;
	
    return Out;
}


// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
float4 PSScreen(float2 Tex : TEXCOORD0) : COLOR0
{
	float4 FullScreenImage = tex2D(FullResMapSampler, Tex);
	float4 BlurredImage = tex2D(RenderMapSampler, Tex);
	
	float4 color = lerp(FullScreenImage, BlurredImage, 0.4f);
	
	color *= ExposureLevel;		    // apply simple exposure	
    return pow(color, 0.95f);
}

//////////////////////////////////////////////////////////////////////////

technique ScaleBuffer
{
    pass P0
    {
        VertexShader = compile vs_1_1 VSScaleBuffer();
        PixelShader  = compile ps_1_1 PSScaleBuffer();
    }
}

technique GaussX
{
    pass P0
    {
        VertexShader = compile vs_1_1 VSGaussX();
        PixelShader  = compile ps_2_0 PSGaussX();
    }
}

technique GaussY
{
    pass P0
    {
        VertexShader = compile vs_1_1 VSGaussY();
        PixelShader  = compile ps_2_0 PSGaussY();
    }
}


technique Screenblit
{
    pass P0
    {
        VertexShader = compile vs_1_1 VSScreen();
        PixelShader  = compile ps_2_0 PSScreen();
    }
}