// -------------------------------------------------------------
// Ambient Lighting
// 
// Copyright (c) 2003 - 2004 Wolfgang F. Engel (wolf@direct3d.net)
// All rights reserved.
// -------------------------------------------------------------

// -------------------------------------------------------------
// variables that are provided by the application
// -------------------------------------------------------------
float4x4	matWorldViewProj;
float4x4	matWorldView;
float4x4	matProj;
float4x4	matWorld;
//float4		lightVec;
float4		clipPlane;
float		timeOfDay;
texture		AlbedoTexture;

texture		SunMask;
float4		sunColor;


sampler AlbedoSampler = sampler_state
{ 
    Texture		= (AlbedoTexture);
    MipFilter	= NONE; 
    MinFilter	= LINEAR;
    MagFilter	= LINEAR;
    AddressU	= WRAP;
    AddressV	= CLAMP;
};

sampler SunMaskSampler = sampler_state
{
	Texture		= (SunMask);
    MipFilter	= NONE; 
    MinFilter	= LINEAR;
    MagFilter	= LINEAR;
    AddressU	= CLAMP;
    AddressV	= CLAMP;
};


// -------------------------------------------------------------
// Vertex shader output channels
// -------------------------------------------------------------
struct VS_OUTPUT
{
	float4	Position	: POSITION;
	float2	TexCoord	: TEXCOORD0;
//	float3	ClipDist	: TEXCOORD1;
};

struct VS_SUNOUT
{
	float4	Position	: POSITION;
	float2	TexCoord	: TEXCOORD0;
//	float3	ClipDist	: TEXCOORD1;
};

//-----------------------------------------------------------------------------
// Pixel shader output structure
//-----------------------------------------------------------------------------
struct PS_OUTPUT
{
	float4	RGBAColor	: COLOR0;	
};

struct PS_SUNOUT
{
	float4	RGBAColor	: COLOR0;	
};


// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------

VS_OUTPUT SkydomeVS	(	float4			Pos		: POSITION,
						float3			Normal	: NORMAL,
						float2			Tex		: TEXCOORD0
					)
{
	VS_OUTPUT	Output;
	
//	float4 transNormal	= mul(Normal, matWorld);
	
	Output.Position		= mul(Pos, matWorldViewProj);
//	Output.Diffuse		= saturate(dot(transNormal, -lightVec));
	Output.TexCoord.x	= timeOfDay;
	Output.TexCoord.y	= Tex.y;
    
	return Output;
}

VS_SUNOUT SunVS	(	float4			Pos		: POSITION,
					float2			Offset	: TEXCOORD0,
					float2			Tex		: TEXCOORD1
				)
{
	VS_SUNOUT	Output;
	
	// do screen-aligned billboarding (see http://www.eisscholle.de/blog/?postid=7)
	Output.Position		= mul(Pos, matWorldView);
	Output.Position.xy  += Offset;
	Output.Position		= mul(Output.Position, matProj);
	Output.TexCoord		= Tex;
	
	return Output;
}


// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
PS_OUTPUT SkydomePS	(VS_OUTPUT In)
{
	PS_OUTPUT	Output;
	
	Output.RGBAColor = tex2D(AlbedoSampler, In.TexCoord);
//	Output.RGBAColor = float4(1.0f, 1.0f, 1.0f, 1.0f);
	
	return Output;
}

PS_SUNOUT SunPS	(VS_OUTPUT In)
{
	PS_SUNOUT	Output;
	
	Output.RGBAColor = tex2D(SunMaskSampler, In.TexCoord) * sunColor;
	
	return Output;
}


// -------------------------------------------------------------
// Name technique | pass | how to compile 
// -------------------------------------------------------------
technique RenderSkydome
{
    pass P0
    {
        // compiler directives
        VertexShader = compile vs_1_1 SkydomeVS();
        PixelShader  = compile ps_1_1 SkydomePS();
    }
}

technique RenderSun
{
    pass P0
    {
        // compiler directives
        VertexShader = compile vs_1_1 SunVS();
        PixelShader  = compile ps_1_1 SunPS();
    }
}